# -*- coding: utf-8 -*-
"""
nmr-example.py <model coordinate file> <NEF file>
^^^^^^^^^^

Example of wwPDB OneDep API NMR validation service -

:copyright: @wwPDB
:license: Apache 2.0
"""
from __future__ import absolute_import, print_function
import os
import sys
import time

HERE = os.path.abspath(os.path.dirname(__file__))

try:
    from onedep import __apiUrl__
except:
    sys.path.insert(0, os.path.dirname(HERE))
    from onedep import __apiUrl__

from onedep.api.Validate import Validate


def print_(s):
    sys.stdout.write(s)


def readApiKey(filePath):
    apiKey = None
    try:
        fn = os.path.expanduser(filePath)
        with open(fn, 'r') as fp:
            apiKey = fp.read()
    except:
        pass
    return apiKey


def displayStatus(sD, exitOnError=True):
    if 'onedep_error_flag' in sD and sD['onedep_error_flag']:
        print_("OneDep error: %s\n" % sD['onedep_status_text'])
        if exitOnError:
            raise SystemExit()
    else:
        if 'status' in sD:
            print_("OneDep status: %s\n" % sD['status'])


def validateNmr(modelFilePath, nefFilePath):
    """ Example of OneDep API NMR validation service.

        First input argument is the path to the model coordinate file (PDBx/mmCIF format).
        Second input argument is the path to the chemical shifts data file (PDBx/mmCIF format).
    """
    print_("Example validation service for: \n +model file %s\n +NEF file %s\n" % (modelFilePath, nefFilePath))
    #
    USEKEY = os.getenv("ONEDEP_USE_API_KEY") if os.getenv("ONEDEP_USE_API_KEY") else False
    #
    # Check for alternative URL and KEY settings in the environment -
    #
    apiUrl = os.getenv("ONEDEP_API_URL") if os.getenv("ONEDEP_API_URL") else __apiUrl__
    # Flag for mock service for testing -
    mockService = True if os.getenv("ONEDEP_API_MOCK_SERVICE") == "Y" else False

    if USEKEY:
        keyFilePath = os.getenv("ONEDEP_API_KEY_PATH") if os.getenv("ONEDEP_API_KEY_PATH") else "~/.onedep_apikey.jwt"
        apiKey = readApiKey(keyFilePath)
        val = Validate(apiKey=apiKey, apiUrl=apiUrl)
    else:
        val = Validate(apiUrl=apiUrl)
    #
    # Create a new service session -
    #
    print_("Creating new session for NMR validation example\n")
    rD = val.newSession()
    displayStatus(rD)
    #
    # Upload target input files -
    #
    print_("Uploading NMR test files %s and %s\n" % (modelFilePath, nefFilePath))
    rD = val.inputModelXyzFile(modelFilePath)
    rD = val.inputNmrDataNef(nefFilePath)
    displayStatus(rD)
    #
    # Submit service request
    if mockService:
        pD = {}
        pD['worker_test_mode'] = True
        pD['worker_test_duration'] = int(os.getenv("ONEDEP_API_MOCK_DURATION"))
        print_("Using mock service setup %r\n" % pD)
        rD = val.run(**pD)
    else:
        print_("Submitted NMR validation request\n")
        rD = val.run()
    displayStatus(rD)
    #
    #   Poll for service completion -
    #
    it = 0
    sl = 2
    while (True):
        #    Pause -
        it += 1
        pause = it * it * sl
        time.sleep(pause)
        rD = val.getStatus()
        if rD['status'] in ['completed', 'failed']:
            break
        print_("[%4d] Pausing for %4d (seconds)\n" % (it, pause))
        #
    #
    # Fetch output validation report and data files
    lt = time.strftime("%Y%m%d%H%M%S", time.localtime())
    if rD['status'] == 'failed':
        print_("Failed to produce report\n")
        fnR = "nmr-report-%s.log" % lt
        rD = val.getReportLog(fnR)
        print_("Fetching report files %s\n" % fnR)
    else:
        fnR = "nmr-report-%s.pdf" % lt
        fnD = "nmr-report-%s.xml" % lt
        print_("Fetching report files %s and %s\n" % (fnR, fnD))
        rD = val.getReport(fnR)
        displayStatus(rD)
        rD = val.getReportData(fnD)
        displayStatus(rD)
    print_("Completed NMR service example\n")


if __name__ == '__main__':

    if len(sys.argv) < 3:
        # Use example data if none are provided -
        modelFilePath = os.path.join(HERE, "data", "7jia.cif.gz")
        nefFilePath = os.path.join(HERE, "data", "7jia_nmr-data.nef.gz")
    else:
        modelFilePath = sys.argv[1]
        nefFilePath = sys.argv[2]
    #
    validateNmr(modelFilePath, nefFilePath)
