# -*- coding: utf-8 -*-
"""
xray-example.py <model coordinate file> <structure factor file>
^^^^^^^^^^

Example of wwPDB OneDep API X-ray validation service -

:copyright: @wwPDB
:license: Apache 2.0
"""
from __future__ import absolute_import, print_function
import os
import sys
import time

HERE = os.path.abspath(os.path.dirname(__file__))

try:
    from onedep import __apiUrl__
except:
    sys.path.insert(0, os.path.dirname(HERE))
    from onedep import __apiUrl__

from onedep.api.Validate import Validate


def print_(s):
    sys.stdout.write(s)


def readApiKey(filePath):
    apiKey = None
    try:
        fn = os.path.expanduser(filePath)
        with open(fn, 'r') as fp:
            apiKey = fp.read()
    except:
        pass
    return apiKey


def displayStatus(sD, exitOnError=True):
    if 'onedep_error_flag' in sD and sD['onedep_error_flag']:
        print_("OneDep error: %s\n" % sD['onedep_status_text'])
        if exitOnError:
            raise SystemExit()
    else:
        if 'status' in sD:
            print_("OneDep status: %s\n" % sD['status'])


def validateXray(modelFilePath, sfFilePath):
    """ Example of OneDep API X-ray validation service.

        First input argument is the path to the model coordinate file (PDBx/mmCIF format).
        Second input argument is the path to the structure factor file (PDBx/mmCIF format).
    """
    print_("Example validation service for: \n +model file %s\n +structure factor file %s\n" % (modelFilePath, sfFilePath))
    #
    USEKEY = os.getenv("ONEDEP_USE_API_KEY") if os.getenv("ONEDEP_USE_API_KEY") else False
    #
    # Check for alternative URL and KEY settings in the environment -
    #
    apiUrl = os.getenv("ONEDEP_API_URL") if os.getenv("ONEDEP_API_URL") else __apiUrl__
    # Flag for mock service for testing -
    mockService = True if os.getenv("ONEDEP_API_MOCK_SERVICE") == "Y" else False
    if USEKEY:
        keyFilePath = os.getenv("ONEDEP_API_KEY_PATH") if os.getenv("ONEDEP_API_KEY_PATH") else "~/.onedep_apikey.jwt"
        apiKey = readApiKey(keyFilePath)
        val = Validate(apiKey=apiKey, apiUrl=apiUrl)
    else:
        val = Validate(apiUrl=apiUrl)
    #
    # Create a new service session -
    #
    print_("Creating new session for X-ray validation example\n")
    rD = val.newSession()
    displayStatus(rD)
    #
    # Upload target input files -
    #
    print_("Uploading X-ray test files %s and %s\n" % (modelFilePath, sfFilePath))
    rD = val.inputModelXyzFile(modelFilePath)
    rD = val.inputStructureFactorFile(sfFilePath)
    displayStatus(rD)
    #
    # Submit service request
    if mockService:
        pD = {}
        pD['worker_test_mode'] = True
        pD['worker_test_duration'] = int(os.getenv("ONEDEP_API_MOCK_DURATION"))
        print_("Using mock service setup %r\n" % pD)
        rD = val.run(**pD)
    else:
        print_("Submitted X-ray validation request\n")
        rD = val.run()
    displayStatus(rD)
    #
    #   Poll for service completion -
    #
    it = 0
    sl = 2
    while (True):
        #    Pause -
        it += 1
        pause = it * it * sl
        time.sleep(pause)
        rD = val.getStatus()
        if rD['status'] in ['completed', 'failed']:
            break
        print_("[%4d] Pausing for %4d (seconds)\n" % (it, pause))
        #
    #
    # Fetch output validation report and data files
    lt = time.strftime("%Y%m%d%H%M%S", time.localtime())
    fnR = "xray-report-%s.pdf" % lt
    fnD = "xray-report-%s.xml" % lt
    print_("Fetching report files %s and %s\n" % (fnR, fnD))
    rD = val.getReport(fnR)
    displayStatus(rD)
    rD = val.getReportData(fnD)
    displayStatus(rD)
    print_("Completed\n")


if __name__ == '__main__':

    if len(sys.argv) < 3:
        # Use example data if none are provided -
        modelFilePath = os.path.join(HERE, "data", "1cbs.cif.gz")
        sfFilePath = os.path.join(HERE, "data", "1cbs-sf.cif.gz")
    else:
        modelFilePath = sys.argv[1]
        sfFilePath = sys.argv[2]
    #
    validateXray(modelFilePath, sfFilePath)
